@@ -3,7 +3,7 @@ package Net::FullAuto::ISets::Local::Chaining_is;
 ### OPEN SOURCE LICENSE - GNU AFFERO PUBLIC LICENSE Version 3.0 #######
 #
 #    Net::FullAuto - Powerful Network Process Automation Software
-#    Copyright © 2000-2015  Brian M. Kelly
+#    Copyright © 2000-2016  Brian M. Kelly
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU Affero General Public License as
@@ -42,6 +42,7 @@ use Net::FullAuto::FA_Core qw[cmd_raw $localhost];
 use File::HomeDir;
 my $home_dir=File::HomeDir->my_home.'/';
 my $username=getlogin || getpwuid($<);
+my $do;my $ad;my $prompt;
 
 my $configure_chaining=sub {
 
@@ -50,8 +51,8 @@ my $configure_chaining=sub {
    my $handle=$localhost;my $connect_error='';
    $localhost->cwd('~');
    my $sudo=($^O eq 'cygwin')?'':'sudo ';
-   my $do=1;
-if ($do) {
+$do=1;
+if ($do==1) {
    unless ($^O eq 'cygwin') {
       ($stdout,$stderr)=$handle->cmd("sudo yum clean all");
       ($stdout,$stderr)=$handle->cmd("sudo yum grouplist hidden");
@@ -63,7 +64,7 @@ if ($do) {
          ' libicu cyrus-sasl-devel libtool-ltdl-devel libxml2-devel'.
          ' freetype-devel libpng-devel java-1.7.0-openjdk-devel'.
          ' unixODBC unixODBC-devel libtool-ltdl libtool-ltdl-devel'.
-         ' ncurses-devel xmlto','__display__');
+         ' ncurses-devel xmlto git-all','__display__');
    } else {
       my $cygcheck=`/bin/cygcheck -c` || die $!;
       my $uname=`/bin/uname` || die $!;
@@ -73,7 +74,7 @@ if ($do) {
       if ($uname_all=~/x86_64/) {
          foreach my $package ('libxml2','libxml2-devel','libtool',
                'autoconf','autobuild','automake','pkg-config',
-               'libuuid-devel','wget') {
+               'libuuid-devel','wget','git') {
             unless (-1<index $cygcheck, "$package ") {
                $need_packages{$package}='';
             }
@@ -81,7 +82,7 @@ if ($do) {
       } else {
          foreach my $package ('libxml2','libxml2-devel','libtool',
                'autoconf','autobuild','automake','pkg-config',
-               'libuuid-devel','wget') {
+               'libuuid-devel','wget','git') {
             unless (-1<index $cygcheck, "$package ") {
                $need_packages{$package}='';
             }
@@ -143,7 +144,7 @@ if ($do) {
    #($stdout,$stderr)=$handle->cmd("rm -rvf $rmqtar",'__display__');
    #($stdout,$stderr)=$handle->cwd($rmqdir);
    #$handle->{_cmd_handle}->print('sudo su');
-   #my $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
+   #$prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    #while (1) {
    #   my $output.=Net::FullAuto::FA_Core::fetch($handle);
    #   last if $output=~/$prompt/;
@@ -224,17 +225,30 @@ if ($do) {
       ($stdout,$stderr)=$handle->cmd("${sudo}ldconfig");
    }
 }
-$do=0;
-if ($do) {
+$do=1;
+if ($do==1) {
+   my $zmq='zeromq-4.1.3.tar.gz';
    ($stdout,$stderr)=$handle->cmd(
       "wget --random-wait --progress=dot ".
-      "http://download.zeromq.org/zeromq-4.1.3.tar.gz",
+      "http://download.zeromq.org/$zmq",
       '__display__');
-   ($stdout,$stderr)=$handle->cmd('tar zxvf zeromq-4.1.3.tar.gz',
+   ($stdout,$stderr)=$handle->cmd("wget -qO- ".
+      "http://download.zeromq.org/SHA1SUMS");
+   my $zmqsha1=$stdout;
+   $zmqsha1=~s/^.*gz\s+(.*?)\s+$zmq.*$/$1/s;
+   ($stdout,$stderr)=$handle->cmd("sha1sum -c - <<<\"$zmqsha1 $zmq\"",
       '__display__');
-   ($stdout,$stderr)=$handle->cmd('rm -rvf zeromq-4.1.3.tar.gz',
-      '__display__');
-   ($stdout,$stderr)=$handle->cwd('zeromq-4.1.3');
+   unless ($stderr) {
+      print(qq{ + CHECKSUM Test for $zmq *PASSED* \n});
+   } else {
+      ($stdout,$stderr)=$handle->cmd("sudo rm -rvf $zmq",'__display__');
+      print "FATAL ERROR! : CHECKSUM Test for $zmq *FAILED* ";
+      &Net::FullAuto::FA_Core::cleanup;
+   }
+   ($stdout,$stderr)=$handle->cmd("tar zxvf $zmq",'__display__');
+   ($stdout,$stderr)=$handle->cmd("rm -rvf $zmq",'__display__');
+   $zmq=~s/\.tar\.gz$//;
+   ($stdout,$stderr)=$handle->cwd($zmq);
    cmd_raw($handle,'export PKG_CONFIG_PATH=/usr/local/lib/pkgconfig');
    ($stdout,$stderr)=$handle->cmd('./configure','__display__');
    if ($^O eq 'cygwin') {
@@ -253,6 +267,55 @@ if ($do) {
    ($stdout,$stderr)=$handle->cmd('make','__display__');
    ($stdout,$stderr)=$handle->cmd("${sudo}make install",'__display__');
    ($stdout,$stderr)=$handle->cwd("..");
+}
+$do=0;
+if ($do==1) {
+   my $go=$1;my $gosha1=$2;
+   ($stdout,$stderr)=$handle->cmd("wget -qO- https://golang.org/dl");
+   if ($^O eq 'cygwin') {
+      $stdout=~
+         /^.*?href=["]([^"]+windows-amd64.zip)["].*?[<]tt[>](.*?)[<].*$/s;
+      $go=$1;$gosha1=$2;
+   } else {
+      $stdout=~
+         /^.*?href=["]([^"]+linux-amd64.tar.gz)["].*?[<]tt[>](.*?)[<].*$/s;
+      $go=$1;$gosha1=$2;
+   }
+   ($stdout,$stderr)=$handle->cmd(
+      "wget --random-wait --progress=dot ".$go,
+      '__display__');
+   $go=~s/^.*\/(.*)$/$1/;
+   ($stdout,$stderr)=$handle->cmd("sha1sum -c - <<<\"$gosha1 $go\"",
+      '__display__');
+   unless ($stderr) {
+      print(qq{ + CHECKSUM Test for $go *PASSED* \n});
+   } else {
+      ($stdout,$stderr)=$handle->cmd("sudo rm -rvf $go",'__display__');
+      print "FATAL ERROR! : CHECKSUM Test for $go *FAILED* ";
+      &Net::FullAuto::FA_Core::cleanup;
+   }
+   if ($^O eq 'cygwin') {
+      ($stdout,$stderr)=$handle->cmd("unzip $go",'__display__');
+   } else {
+      ($stdout,$stderr)=$handle->cmd("tar zxvf $go",'__display__');
+   }
+   ($stdout,$stderr)=$handle->cmd("rm -rvf $go",'__display__');
+}
+$do=0;
+if ($do==1) {
+   ($stdout,$stderr)=$handle->cmd('wget -qO- '.
+      'https://github.com/membrane/service-proxy/releases/latest');
+   $stdout=~s/^.*?href=["]([^"]+zip)["].*$/$1/s;
+   my $membrane_zip=$stdout;
+   ($stdout,$stderr)=$handle->cmd(
+      "wget --random-wait --progress=dot https://github.com".$membrane_zip,
+      '__display__');
+   $membrane_zip=~s/^.*\/(.*)$/$1/;
+   ($stdout,$stderr)=$handle->cmd("unzip $membrane_zip",'__display__');
+   ($stdout,$stderr)=$handle->cmd("rm -rvf $membrane_zip",'__display__');
+   #($stdout,$stderr)=$handle->cmd('git clone --depth=1 '.
+   #   'https://github.com/membrane/service-proxy.git','__display__');
+exit;
 }
    if ($^O eq 'cygwin') {
       ($stdout,$stderr)=$handle->cwd("~");
@@ -262,7 +325,7 @@ if ($do) {
          '__display__');
       $handle->{_cmd_handle}->print('sudo cpan');
    } 
-   my $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
+   $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    while (1) {
       my $output.=Net::FullAuto::FA_Core::fetch($handle);
       last if $output=~/$prompt/;
@@ -523,6 +586,18 @@ END
       '__display__');
    print "\n";
    $show=<<END;
+
+########################################
+
+   INSTALLING CatalystX::OAuth2
+
+########################################
+END
+   cmd_raw($handle,"${sudo}cpan CatalystX::OAuth2",
+      '__display__');
+   print "\n";
+   $show=<<END;
+
 ########################################
 
    INSTALLING Task::Catalyst::Tutorial
@@ -732,6 +807,45 @@ END
    ($stdout,$stderr)=$handle->cwd("../../..");
    ($stdout,$stderr)=$handle->cmd("./script/adventrest_create.pl ".
       "model DB DBIC::Schema AdventREST::Schema",'__display__');
+   ($stdout,$stderr)=$handle->cmd("./script/adventrest_create.pl ".
+      "controller OAuth2::Provider",'__display__');
+   my $pro_path="./lib/AdventREST/Controller/OAuth2/Provider.pm";
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i ".
+      "\"s/Catalyst::Controller/Catalyst::Controller::ActionRole/\" ".
+      $pro_path);
+   $ad="%NL%".
+       "with %SQ%CatalystX::OAuth2::Controller::Role::Provider%SQ%;%NL%".
+       "%NL%".
+       "__PACKAGE__->config(%NL%".
+       "  store => {%NL%".
+       "    class => %SQ%DBIC%SQ%,%NL%".
+       "    client_model => %SQ%DB::Client%SQ%%NL%".
+       "  }%NL%".
+       ");%NL%".
+       "%NL%".
+       "sub request : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::RequestAuth%SQ%) {}%NL%".
+       "%NL%".
+       "sub grant : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::GrantAuth%SQ%) {%NL%".
+       "  my ( \$self, \$c ) = \@_;%NL%".
+       "%NL%".
+       "  my \$oauth2 = \$c->req->oauth2;%NL%".
+       "%NL%".
+       "  \$c->user_exists and \$oauth2->user_is_valid(1)%NL%".
+       "    or \$c->detach(%SQ%/passthrulogin%SQ%);%NL%".
+       "}%NL%".
+       "%NL%".
+       "sub token : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::AuthToken::ViaAuthGrant%SQ%) {}%NL%".
+       "%NL%".
+       "sub refresh : Chained(%SQ%/%SQ%) Args(0) ".
+       "Does(%SQ%OAuth2::AuthToken::ViaRefreshToken%SQ%) {}%NL%";
+   ($stdout,$stderr)=$handle->cmd(
+      "${sudo}sed -i '/ActionRole/a$ad' $pro_path");
+   ($stdout,$stderr)=$handle->cmd( # bash shell specific
+      "${sudo}sed -i \'s/%NL%/\'\"`echo \\\\\\n`/g\" $pro_path");
+   ($stdout,$stderr)=$handle->cmd("${sudo}sed -i \"s/%SQ%/\'/g\" $pro_path");
    $content=<<'END';
 name: AdventREST
 Model::DB:
@@ -970,7 +1084,7 @@ END
    ($stdout,$stderr)=$handle->cmd("touch User.pm");
    ($stdout,$stderr)=$handle->cmd("chmod -v 777 User.pm",'__display__');
    ($stdout,$stderr)=$handle->cmd("echo -e \"$content\" > User.pm");
-   my $ad='sub index : Private {%NL%   my ( $self, $c ) = @_;%NL%'.
+   $ad='sub index : Private {%NL%   my ( $self, $c ) = @_;%NL%'.
           '   $c->forward( $c->view(%SQ%TT%SQ%) );%NL%}%NL%%NL%';
    ($stdout,$stderr)=$handle->cmd(
       "${sudo}sed -i '/sub index :Path/i$ad' ./Root.pm");
@@ -1423,7 +1537,7 @@ END
           |_| \_|\___|\__|     |_|   \__,_|_|_/_/   \_\__,_|\__\___/ ©
 
 
-   Copyright © 2000-2015  Brian M. Kelly  Brian.Kelly@FullAuto.com
+   Copyright © 2000-2016  Brian M. Kelly  Brian.Kelly@FullAuto.com
 
 END
    eval {
